<?php
/**
 * This file is part of a project maintained at  NC State University.
 *
 * More information is included in the project README.md.
 *
 * (c) NC State University
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace NCSU\ItecsDuskMacros;

use Laravel\Dusk\Browser;
use Facebook\WebDriver\WebDriverBy;
use Facebook\WebDriver\WebDriverKeys;

class DuskMacros
{
    public static function toArray()
    {
        return [
            'consoleLogs' => function () {
                return $this->driver->manage()->getLog('browser');
            },
            'consoleLogErrors' => function () {
                return array_filter($this->driver->manage()->getLog('browser'), function ($entry) {
                    return $entry['level'] != 'INFO';
                });
            },
            'diagnosticOutput' => function ($output) {
                fwrite(STDERR, print_r($output, TRUE));
            },
            'dumpConsoleLogs' => function () {
                print_r($this->driver->manage()->getLog('browser'));
                return $this;
            },
            'eraseField' => function ($selector) {
                $value = $this->value($selector);
                $this->keys($selector, ...array_fill(0, strlen($value), '{backspace}'));
                return $this;
            },
            'fillHiddenField' => function ($name, $value) {
                $this->script("document.getElementsByName('$name')[0].value = '$value'");
                return $this;
            },
            'optionXPathByText' => function ($labelText) {
                return sprintf('.//div[contains(@class,"menuable__content__active")]//div[text()="%s"][@class="v-list-item__title"]', $labelText);
            },
            'sendSpaceToCheckbox' => function ($selector) {
                $this->element($selector)->sendKeys(WebDriverKeys::SPACE);
                return $this;
            },
            'vAutocompleteOption' => function ($vSelectSelector, $labelText) {
                $this->click($vSelectSelector)
                    ->waitFor('.menuable__content__active')
                    ->driver
                    ->findElement(WebDriverBy::xpath($this->optionXPathByText($labelText)))
                    ->click();
                return $this;
            },
            'vCheckbox' => function ($vToggleSelector) {
                if ($vToggleSelector != ltrim($vToggleSelector, '@')) {
                    // dusk selector?
                    $xpath = sprintf('.//input[contains(@role,"checkbox")][@dusk="%s"]/following-sibling::div[@class="v-input--selection-controls__ripple"]', ltrim($vToggleSelector, '@'));
                } elseif ($vToggleSelector != ltrim($vToggleSelector, '.')) {
                    // class selector?
                    $xpath = sprintf('.//input[contains(@role,"checkbox")][@class="%s"]/following-sibling::div[@class="v-input--selection-controls__ripple"]', ltrim($vToggleSelector, '.'));
                } elseif ($vToggleSelector != ltrim($vToggleSelector, '#')) {
                    // id selector?
                    $xpath = sprintf('.//input[contains(@role,"checkbox")][@id="%s"]/following-sibling::div[@class="v-input--selection-controls__ripple"]', ltrim($vToggleSelector, '#'));
                } else {
                    // as a fallback, assume it is a raw xpath predicate
                    $xpath = sprintf('.//input[contains(@role,"checkbox")][%s]/following-sibling::div[@class="v-input--selection-controls__ripple"]', $vToggleSelector);
                }
                $this->driver->findElement(WebDriverBy::xpath($xpath))->click();
                return $this;
            },
            'vSelectOption' => function ($vSelectSelector, $labelText) {
                $this->click($vSelectSelector)
                    ->driver
                    ->findElement(WebDriverBy::xpath($this->optionXPathByText($labelText)))
                    ->click();
                return $this;
            },
            'vToggle' => function ($vToggleSelector) {
                if ($vToggleSelector != ltrim($vToggleSelector, '@')) {
                    // dusk selector?
                    $xpath = sprintf('.//input[contains(@role,"switch")][@dusk="%s"]/following-sibling::div[@class="v-input--selection-controls__ripple"]', ltrim($vToggleSelector, '@'));
                } elseif ($vToggleSelector != ltrim($vToggleSelector, '.')) {
                    // class selector?
                    $xpath = sprintf('.//input[contains(@role,"switch")][@class="%s"]/following-sibling::div[@class="v-input--selection-controls__ripple"]', ltrim($vToggleSelector, '.'));
                } elseif ($vToggleSelector != ltrim($vToggleSelector, '#')) {
                    // id selector?
                    $xpath = sprintf('.//input[contains(@role,"switch")][@id="%s"]/following-sibling::div[@class="v-input--selection-controls__ripple"]', ltrim($vToggleSelector, '#'));
                } else {
                    // as a fallback, assume it is a raw xpath predicate
                    $xpath = sprintf('.//input[contains(@role,"switch")][%s]/following-sibling::div[@class="v-input--selection-controls__ripple"]', $vToggleSelector);
                }
                $this->driver->findElement(WebDriverBy::xpath($xpath))->click();
                return $this;
            },
            'waitAndClickOn' => function ($selector) {
                return $this->waitFor($selector)->click($selector);
            },
        ];
    }

    public static function register(): void
    {
        foreach (self::toArray() as $name => $func) {
            if (!is_string($name)) {
                throw new \InvalidArgumentException('Macro name ' . $name . ' is not a string');
            }
            if (!is_callable($func)) {
                throw new \InvalidArgumentException('Macro argument ' . $name . ' was not callable');
            }
            Browser::macro($name, $func);
        }
    }
}
